///--------------------------------------------------------------------------------------
//	SpriteWorldUtils.c
//
//	Portions are copyright:  1991-94 Tony Myles, All rights reserved worldwide.
//
//	Description:	some utilities for creating worlds of sprites
///--------------------------------------------------------------------------------------


#ifndef __SWCOMMON__
#include "SWCommonHeaders.h"
#endif

#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif

#ifndef __QDOFFSCREEN__
#include <QDOffscreen.h>
#endif

#ifndef __WINDOWS__
#include <Windows.h>
#endif

#ifndef __MEMORY__
#include <Memory.h>
#endif

#ifndef __GESTALTEQU__
#include <GestaltEqu.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __SPRITEWORLD__
#include "SpriteWorld.h"
#endif

#ifndef __SPRITELAYER__
#include "SpriteLayer.h"
#endif

#ifndef __SPRITE__
#include "Sprite.h"
#endif

#ifndef __SPRITEFRAME__
#include "SpriteFrame.h"
#endif

#ifndef __SPRITEWORLDUTILS__
#include "SpriteWorldUtils.h"
#endif


static OSErr				gSWStickyError;

pascal Boolean blackenColorSearch( 
	RGBColor *RGB, long *position );

///--------------------------------------------------------------------------------------
// SWCreateRegionFromCIconMask
///--------------------------------------------------------------------------------------

SW_FUNC OSErr SWCreateRegionFromCIconMask(
	RgnHandle *maskRgn,
	CIconHandle cIconH)
{
	OSErr err = noErr;
	RgnHandle tempMaskRgn;
	char saveState;
	BitMap iconMask;

	*maskRgn = NULL;

	saveState = HGetState((Handle)cIconH);
	HLock((Handle)cIconH);

	iconMask.rowBytes = (**cIconH).iconMask.rowBytes;
	iconMask.bounds = (**cIconH).iconMask.bounds;
	iconMask.baseAddr = (Ptr)(**cIconH).iconMaskData;

	tempMaskRgn = NewRgn();

	if (tempMaskRgn != NULL)
	{
		err = BitMapToRegion(tempMaskRgn, &iconMask);

		if (err == noErr)
		{
			*maskRgn = tempMaskRgn;
		}
		else
		{
			DisposeRgn(tempMaskRgn);
		}
	}
	else
	{
		err = MemError();
	}

	HSetState((Handle)cIconH, saveState);

	SWSetStickyIfError( err );
	return err;
}



///--------------------------------------------------------------------------------------
// SWCreateRegionFromPict
///--------------------------------------------------------------------------------------

SW_FUNC OSErr SWCreateRegionFromPict(
	RgnHandle *pictRgnH,
	PicHandle srcPictH)
{
	RgnHandle 			tempRgnH;
	GWorldPtr 			saveGWorld, 
						offScrnGWld;
	GDHandle			saveGDH;
	Rect				frameRect;
	ColorSearchUPP		blackenColSearchUPP;
	OSErr 				err;


	*pictRgnH = NULL;
	blackenColSearchUPP = NewColorSearchProc( blackenColorSearch );
	
	GetGWorld(&saveGWorld, &saveGDH );
	
	frameRect = (**srcPictH).picFrame;
	OffsetRect(&frameRect, -frameRect.left, -frameRect.top);
		
	err = NewGWorld( &offScrnGWld, 1, &frameRect, nil, nil, 0 );
	
	if (err == noErr)
	{	
		(void)LockPixels( GetGWorldPixMap(offScrnGWld) );
		SetGWorld(offScrnGWld, nil);
		AddSearch( blackenColSearchUPP );
		EraseRect(&frameRect);
		DrawPicture(srcPictH, &frameRect);
		DelSearch( blackenColSearchUPP );
#if SW_PPC
		DisposeRoutineDescriptor( blackenColSearchUPP );
#endif
		tempRgnH = NewRgn();

		if (tempRgnH != NULL && err == noErr)
		{
			err = BitMapToRegion(tempRgnH, (BitMap*)&offScrnGWld->portPixMap);
			
			UnlockPixels( GetGWorldPixMap(offScrnGWld) );
			
			if (err == noErr)
			{
				*pictRgnH = tempRgnH;
			}
			else
			{
				DisposeRgn(tempRgnH);
			}
		}
		else
		{
			if ( err == noErr )
				err = MemError();
		}

		SetGWorld(saveGWorld, nil);
		DisposeGWorld(offScrnGWld);
	}

	SWSetStickyIfError( err );
	return err;
}


///--------------------------------------------------------------------------------------
// SWCreateRegionFromGWorldAndRect
///--------------------------------------------------------------------------------------

SW_FUNC OSErr SWCreateRegionFromGWorldAndRect(
	RgnHandle *maskRgn,
	GWorldPtr maskGWorld,
	Rect* frameRect)
{
	OSErr 			err = noErr;
	GDHandle		saveGDH;
	GWorldPtr 		saveGWorld, 
					tempMaskGWorld;
	Rect			tempMaskRect;
	RgnHandle 		tempRgnH;
	

	*maskRgn = NULL;
	
	GetGWorld(&saveGWorld, &saveGDH);

	tempMaskRect = *frameRect;
	OffsetRect(&tempMaskRect, -tempMaskRect.left, -tempMaskRect.top);
	
	err = NewGWorld( &tempMaskGWorld, 1, &tempMaskRect, nil, nil, 0 );
	
	if (err == noErr)
	{
		(void)LockPixels( GetGWorldPixMap(tempMaskGWorld) );
		(void)LockPixels( GetGWorldPixMap(maskGWorld) );
		SetGWorld(tempMaskGWorld, nil);
		
		CopyBits( (BitMap*)*GetGWorldPixMap( maskGWorld ), 
			(BitMap*)*GetGWorldPixMap( tempMaskGWorld ), 
			frameRect, 
			&tempMaskRect,
			srcCopy, 
			nil );

		tempRgnH = NewRgn();
		if (tempRgnH != NULL)
		{
			err = BitMapToRegion(tempRgnH, (BitMap*)&tempMaskGWorld->portPixMap);
						
			UnlockPixels( GetGWorldPixMap(tempMaskGWorld) );
			UnlockPixels( GetGWorldPixMap(maskGWorld) );
			
			if (err == noErr)
			{
				*maskRgn = tempRgnH;
			}
			else
			{
				DisposeRgn(tempRgnH);
			}
		}
		else
		{
			err = MemError();
		}
		SetGWorld(saveGWorld, nil);
		DisposeGWorld(tempMaskGWorld);
	}
	
	SWSetStickyIfError( err );
	return err;
}

///--------------------------------------------------------------------------------------
// SWCreateGWorldFromPictResource
///--------------------------------------------------------------------------------------
SW_FUNC OSErr SWCreateGWorldFromPictResource(
	SpriteWorldPtr destSpriteWorld,
	GWorldPtr *pictGWorldP,
	short pictResID)
{
	OSErr 			err = noErr;
	PicHandle 		newPictH;
	
	
	newPictH = GetPicture(pictResID);

	if (newPictH != NULL)
	{
		err = SWCreateGWorldFromPict(destSpriteWorld, pictGWorldP, newPictH);

		ReleaseResource((Handle)newPictH);
	}
	else
	{
		err = MemError();
		if (err == noErr)
		{
			err = resNotFound;
		}
	}

	SWSetStickyIfError( err );
	return err;
}


///--------------------------------------------------------------------------------------
// SWCreateGWorldFromPict
//	creates a offScreen GWorld and draws the specified pict into it
///--------------------------------------------------------------------------------------
SW_FUNC OSErr SWCreateGWorldFromPict(
	SpriteWorldPtr destSpriteWorld,
	GWorldPtr *pictGWorld,
	PicHandle pictH)
{
	OSErr				err;
	GWorldPtr 		saveGWorld;
	GDHandle 		saveGDevice;
	GWorldPtr 		tempGWorld;
	Rect 				pictRect;
	short				depth;
	GDHandle			theGDH;


	*pictGWorld = NULL;
		
	GetGWorld(&saveGWorld, &saveGDevice);
	
	depth = destSpriteWorld->pixelDepth;
	theGDH = destSpriteWorld->mainSWGDH;

	pictRect = (**pictH).picFrame;
	OffsetRect(&pictRect, -pictRect.left, -pictRect.top);

	err = NewGWorld( &tempGWorld, depth, &pictRect, nil, 
				theGDH, noNewDevice );

	if (err == noErr)
	{	
		*pictGWorld = tempGWorld;

		SetGWorld(tempGWorld, nil);

		(void)LockPixels( GetGWorldPixMap(tempGWorld) );
		EraseRect(&pictRect);
		DrawPicture(pictH, &pictRect);
		UnlockPixels( GetGWorldPixMap(tempGWorld) );
	}

	SetGWorld(saveGWorld, saveGDevice);
	
	SWSetStickyIfError( err );
	return err;
}


///--------------------------------------------------------------------------------------
// SWCreateGWorldFromCIconMask
///--------------------------------------------------------------------------------------

SW_FUNC OSErr SWCreateGWorldFromCIconMask(
	SpriteWorldPtr destSpriteWorld,
	GWorldPtr *maskGWorldP,
	CIconHandle cIconH)
{
	OSErr 			err;
	GWorldPtr 		saveGWorld;
	GDHandle 		saveGDevice;
	char 			saveState;
	BitMap 			maskBitMap;
	GWorldPtr 		tempGWorldP;
	short			depth;
	GDHandle		theGDH;


	GetGWorld(&saveGWorld, &saveGDevice);

	depth = destSpriteWorld->pixelDepth;
	theGDH = destSpriteWorld->mainSWGDH;
	
	saveState = HGetState((Handle)cIconH);
	HLock((Handle)cIconH);

	maskBitMap.rowBytes = (**cIconH).iconMask.rowBytes;
	maskBitMap.bounds = (**cIconH).iconMask.bounds;
	maskBitMap.baseAddr = (Ptr)(**cIconH).iconMaskData;
	
	err = NewGWorld( &tempGWorldP, depth, &maskBitMap.bounds, nil, 
						theGDH, noNewDevice );
	
	if (err == noErr)
	{	
		SetGWorld(tempGWorldP, NULL);
		(void)LockPixels( GetGWorldPixMap(tempGWorldP) );
		CopyBits(&maskBitMap, 
			(BitMap*)*GetGWorldPixMap( tempGWorldP ),
			&maskBitMap.bounds, 
			&maskBitMap.bounds,
			srcCopy, 
			nil);		
		UnlockPixels( GetGWorldPixMap(tempGWorldP) );
		
		*maskGWorldP = tempGWorldP;
	}

	SetGWorld(saveGWorld, saveGDevice);

	HSetState((Handle)cIconH, saveState);

	SWSetStickyIfError( err );
	return err;
}

///--------------------------------------------------------------------------------------
//	SWBlackenGWorld
///--------------------------------------------------------------------------------------
SW_FUNC OSErr SWBlackenGWorld( GWorldPtr oldGWorld )
{
	GWorldPtr			saveGWorld, newGWld;
	GDHandle			saveGDH;
	Rect				gwldRect;
	GWorldFlags			pixelState;
	ColorSearchUPP		blackenColSearchUPP;
	OSErr				err;
	
	
	blackenColSearchUPP = NewColorSearchProc( blackenColorSearch );
		
	GetGWorld( &saveGWorld, &saveGDH );
	
	gwldRect = oldGWorld->portRect;
	pixelState = GetPixelsState( oldGWorld->portPixMap );
	
	err = NewGWorld(&newGWld, 1, &gwldRect, nil, nil, 0 );
	if ( err == noErr )
	{
		(void)LockPixels( GetGWorldPixMap(newGWld) );
		(void)LockPixels( GetGWorldPixMap(oldGWorld) );
		SetGWorld( newGWld, nil );
		AddSearch( blackenColSearchUPP );
		CopyBits ( (BitMap*)*GetGWorldPixMap( oldGWorld ),
			(BitMap*)*GetGWorldPixMap( newGWld ),
			&gwldRect, 
			&gwldRect, 
			srcCopy, 
			nil);
		DelSearch( blackenColSearchUPP );
#if SW_PPC
		DisposeRoutineDescriptor( blackenColSearchUPP );
#endif
		SetGWorld( oldGWorld, nil );
		CopyBits ( (BitMap*)*GetGWorldPixMap( newGWld ),
			(BitMap*)*GetGWorldPixMap( oldGWorld ),
			&gwldRect, 
			&gwldRect, 
			srcCopy, 
			nil);
		
		SetPixelsState( oldGWorld->portPixMap, pixelState );
		UnlockPixels( GetGWorldPixMap(newGWld) );
		DisposeGWorld( newGWld );
	}
	
	SetGWorld( saveGWorld, saveGDH );
	
	SWSetStickyIfError( err );
	return err;
}


///--------------------------------------------------------------------------------------
//	blackenColorSearch
///--------------------------------------------------------------------------------------
pascal Boolean blackenColorSearch( RGBColor *RGB, long* position )
{
	if ( !(RGB->red == 0xFFFF && RGB->green == 0xFFFF && RGB->blue == 0xFFFF) )
	{
		RGB->red = RGB->green = RGB->blue = 0x0000;
	}
	return ( false );
}


///--------------------------------------------------------------------------------------
//	SWClearStickyError
///--------------------------------------------------------------------------------------

SW_FUNC void SWClearStickyError(void)
{
	gSWStickyError = 0;
}

///--------------------------------------------------------------------------------------
//	SWStickyError
///--------------------------------------------------------------------------------------

SW_FUNC OSErr SWStickyError(void)
{
	return gSWStickyError;
}

///--------------------------------------------------------------------------------------
//	SWSetStickyIfError
///--------------------------------------------------------------------------------------

SW_FUNC void SWSetStickyIfError(OSErr errNum)
{
	if ( errNum != noErr )
		gSWStickyError = errNum;
}


///--------------------------------------------------------------------------------------
// SWHasSystem7
///--------------------------------------------------------------------------------------

SW_FUNC Boolean SWHasSystem7(void)
{
	OSErr err;
	long	versionNumber;
	
	err = Gestalt( gestaltSystemVersion, &versionNumber );

	SWSetStickyIfError( err );
	return (err == noErr) && (versionNumber >= 0x0700);
}

